==> ContentDispatcher-1.0.1.tgz: ContentDispatcher/tool.py <==
# -*- coding: utf-8 -*-
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import time
import os
import urllib
from urlparse import urlparse, urlunparse

from App.config import getConfiguration

from AccessControl import ClassSecurityInfo
from AccessControl.User import SimpleUser
from AccessControl.SecurityManagement import getSecurityManager
from AccessControl.SecurityManagement import newSecurityManager

from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager

from logging import getLogger

from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from Products.CMFCore import permissions as CCP
from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.Expression import Expression

LOG = getLogger('ContentDispatcher')


class UserAsManager(SimpleUser):
   """
       Give the current user manager role for a specified time.
   """
   def __init__(self, user):
       self.name = '%s_userAsManager_%s' % (user.getUserName(), str(time.time()))
       self.__ = user._getPassword()
       self.domains = user.getDomains()
       roles = list(user.getRoles())
       roles.append('Manager')
       self.roles = tuple(roles)

class WrongAuthenticationError(Exception):
    pass

class CDURLOpener(urllib.FancyURLopener):
    def prompt_user_passwd(self, host, realm):
        ## if we are here, this is because the authentication is wrong.
        raise WrongAuthenticationError()

## see documentation, this is not a hack.
urllib._urlopener = CDURLOpener()

# security.declareProtected(Permission, methodNameAsString)
# security.declarePrivate(methodNameAsString)
# security.declarePublic(methodNameAsString)
# default manager permission is CCP.ManagePortal


class ContentDispatcherTool(PropertyManager, UniqueObject, SimpleItem, ActionProviderBase):
    """
        Configuration and "engine" of Content Dispatcher.
    """
    id = 'content_dispatcher_tool'
    meta_type = 'ContentDispatcherTool'
    plone_tool = 1
    title = "Content Dispatcher Tool"

    _properties = (
        {
            "id" : "distantLogin",
            "type" : "string",
            "mode" : "w",
            "label" : "Login to use",
        },
        {
            "id" : "distantPassword",
            "type" : "string",
            "mode" : "w",
            "label" : "Password to use",
        },
        {
            "id" : "distantServers",
            "type" : "lines",
            "mode" : "w",
            "label" : "URL to use",
        },
        {
            "id" : "incomingFolder",
            "type" : "string",
            "mode" : "w",
            "label" : "Incoming folder",
        },
        {
            "id" : "emailMessage",
            "type" : "text",
            "mode" : "w",
            "label" : "Message sent on content received",
        },

    )
    distantLogin = "admin"
    distantPassword = "pass"
    distantServers = ()
    incomingFolder = ""
    emailMessage = """New content has been imported in your site.
The original content is available at this address: %(sourceURL)s

The new content is available here: %(newURL)s
"""

    fixedSuffix = "/content_dispatcher_tool/receive"

    overviewPath = os.path.join(os.path.dirname(__file__), 'zmi', 'overview')

    manage_options = (

        ({'label' : 'Overview',
          'action' : 'overview'
          },)
        + ActionProviderBase.manage_options
        + PropertyManager.manage_options
        + SimpleItem.manage_options
    )

    security = ClassSecurityInfo()
    overview = PageTemplateFile(overviewPath)
    security.declareProtected(CCP.ManagePortal, 'overview')

    _actions = (
        ActionInformation(
            id='dispatch',
            title='Dispatch',
            action=Expression(
                text='python:"%s/cd_dispatch" % (object.isDefaultPageInFolder() and object.getParentNode().absolute_url() or object_url)'
            ),
            condition=Expression(
                text="python: object." + id + ".isActionDisplayed(object)"
            ),
            permissions=(CCP.ReviewPortalContent,),
            category='object_buttons',
            visible=True,
        ),
    )


    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container) :
        SimpleItem.manage_afterAdd(self, item, container)

    security.declareProtected(CCP.ManagePortal, "getDistantServersAsText")
    def getDistantServersAsText(self):
        return "\n".join(self.distantServers)

    security.declarePrivate('buildURL')
    def buildURL(self, url):
        urlBlocks = list(urlparse(url + self.fixedSuffix))
        urlBlocks[1] = "%s:%s@%s" % (self.distantLogin, self.distantPassword, urlBlocks[1])
        urlToCall = urlunparse(tuple(urlBlocks))
        return urlToCall

    security.declareProtected(CCP.ManagePortal, "sendContent")
    def sendContent(self, url, params):
        """
            Send the content to one of the distant servers.
        """
        try:
            response = urllib.urlopen(self.buildURL(url), params)
        except WrongAuthenticationError:
            raise Exception("Access denied.")
        except IOError, e:
            raise Exception("IOError: %s" % e)

        result = response.read()
        result.strip()
        if result != "ok":
            e = Exception("Server error: %s" % response.info()["Bobo-Exception-Type"])
            e.serverError = response.info()["Bobo-Exception-Type"]
            raise e

    security.declarePublic('getMemberInfo')
    def getMemberInfo(self, memberId=None, info=None):
        """
        Extended version of Portal membership.getMemberInfo
        return more fields:
        - email, company...

        if memberId is NOT given, return info for logged user
        if info is given, return only this info.
        """
        mtool = getToolByName(self, 'portal_membership')
        if not memberId:
            member = mtool.getAuthenticatedMember()
        else:
            member = mtool.getMemberById(memberId)

        if member is None:
            return None

        if info:
            return member.getProperty(info)

        memberinfo = {
            'fullname'    : member.getProperty('fullname'),
            'description' : member.getProperty('description'),
            'location' : member.getProperty('location'),
            'language' : member.getProperty('language'),
            'home_page' : member.getProperty('home_page'),
            'email': member.getProperty('email'),
            'company': member.getProperty('company', ''),
            'area': member.getProperty('area', ''),
            'country': member.getProperty('country', ''),
            'job': member.getProperty('job', ''),
        }

        return memberinfo


    security.declareProtected(CCP.ReviewPortalContent, "dispatch")
    def dispatch(self, context):
        """
            Dispatch the context to the specified servers
        """

        portal_properties = getToolByName(self, "portal_properties")
        if context.portal_type in portal_properties.site_properties.typesUseViewActionInListings:
            objectUrl = context.absolute_url() + "/view"
        else:
            objectUrl = context.absolute_url()

        ## update security to allow user to be manager
        currentUser = getSecurityManager().getUser()

        try:
            userAsManager = UserAsManager(currentUser)
            newSecurityManager(self.REQUEST, userAsManager)

            ## download has to be true to enable this method returning
            ## something.
            ## zexpFile is a string coming from a IOString
            zexpFile = context.getParentNode().manage_exportObject(id=context.getId(), download=True)
        finally:
            newSecurityManager(self.REQUEST, currentUser)

        error = False
        dispatch = False
        already = False
        senderEmail = self.getMemberInfo(info="email")

        ## We only check for syntaxically correct args.
        line = 0
        for server in self.distantServers: ## this is a tuple
            line += 1
            try:
                (url, mailAddress) = server.split()
            except ValueError:
                raise RuntimeError("Content dispatcher address lines are not well formated at line %d." % line)

        for server in self.distantServers:
            (url, mailAddress) = server.split()
            dispatch = True
            params = urllib.urlencode(
                {
                    "file": zexpFile,
                    "sourceURL": objectUrl,
                    "objectId": context.getId(),
                    "fromAddr": senderEmail,
                    "toAddr": mailAddress,
                }
            )

            try:
                self.sendContent(url, params)
            except Exception, e:
                if hasattr(e, "serverError") and e.serverError == "BadRequest": ## id already in use
                    already = True
                    continue
                error = True
                LOG.error(str(e) + " (%s)" % url)

        if already:
            message = "Your content has already been dispatched. If not, please report this problem to your administrator."
        elif not dispatch:
            message = "No distant server is specified: nothing to do."
        elif error:
            message = "An error has occured. Please report this problem to your administrator."
        else:
            message = "Content successfully dispatched."
        newUrl = objectUrl + "?portal_status_message=%s" % urllib.quote(message)
        self.REQUEST.RESPONSE.redirect(newUrl)


    security.declareProtected(CCP.ManagePortal, "receive")
    def receive(self):
        """
            Receive a zexp from nother site.
        """
        request = self.REQUEST
        filename = "receivedZexp.zexp"

        cfg = getConfiguration()

        filepath = os.path.join(cfg.instancehome, 'import', filename)
        zfile = file(filepath, "wb")
        zfile.write(request["file"])
        zfile.close()

        folder = self.restrictedTraverse(self.incomingFolder)
        folder.manage_importObject(filename)

        os.remove(filepath)

        mailhost = getToolByName(self, "MailHost")
        mailhost.send(
            self.emailMessage % {
                "sourceURL": request["sourceURL"],
                "newURL": folder.absolute_url() + '/' +  request["objectId"],
            },
            mto=request["toAddr"],
            mfrom=request["fromAddr"],
            subject="New content has arrived!",
        )

        ## don't change this, this is the marker of a well done transaction
        return "ok"

    security.declarePublic(CCP.ManagePortal, "isActionDisplayed")
    def isActionDisplayed(self, context):
        if len(self.distantServers):
            return True
        return False


InitializeClass(ContentDispatcherTool)

==> ContentDispatcher-1.0.2.tgz: ContentDispatcher/tool.py <==
# -*- coding: utf-8 -*-
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import time
import sys, os
import urllib
from urlparse import urlparse, urlunparse
import socket

from App.config import getConfiguration

from AccessControl import ClassSecurityInfo
from AccessControl.User import SimpleUser
from AccessControl.SecurityManagement import getSecurityManager
from AccessControl.SecurityManagement import newSecurityManager

from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager

from logging import getLogger

from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from Products.CMFCore import permissions as CCP
from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.Expression import Expression

LOG = getLogger('ContentDispatcher')


class UserAsManager(SimpleUser):
   """
       Give the current user manager role for a specified time.
   """
   def __init__(self, user):
       self.name = '%s_userAsManager_%s' % (user.getUserName(), str(time.time()))
       #self.__ = user._getPassword()
       self.__ = "wrongPass"
       self.domains = user.getDomains()
       roles = list(user.getRoles())
       roles.append('Manager')
       self.roles = tuple(roles)

class WrongAuthenticationError(Exception):
    pass

class CDURLOpener(urllib.FancyURLopener):
    def prompt_user_passwd(self, host, realm):
        ## if we are here, this is because the authentication is wrong.
        raise WrongAuthenticationError()

## see documentation, this is not a hack.
urllib._urlopener = CDURLOpener()

# security.declareProtected(Permission, methodNameAsString)
# security.declarePrivate(methodNameAsString)
# security.declarePublic(methodNameAsString)
# default manager permission is CCP.ManagePortal


class ContentDispatcherTool(PropertyManager, UniqueObject, SimpleItem, ActionProviderBase):
    """
        Configuration and "engine" of Content Dispatcher.
    """
    id = 'content_dispatcher_tool'
    meta_type = 'ContentDispatcherTool'
    plone_tool = 1
    title = "Content Dispatcher Tool"

    _properties = (
        {
            "id" : "distantLogin",
            "type" : "string",
            "mode" : "w",
            "label" : "Login to use",
        },
        {
            "id" : "distantPassword",
            "type" : "string",
            "mode" : "w",
            "label" : "Password to use",
        },
        {
            "id" : "distantServers",
            "type" : "lines",
            "mode" : "w",
            "label" : "URL to use",
        },
        {
            "id" : "incomingFolder",
            "type" : "string",
            "mode" : "w",
            "label" : "Incoming folder",
        },
        {
            "id" : "emailMessage",
            "type" : "text",
            "mode" : "w",
            "label" : "Message sent on content received",
        },

    )
    distantLogin = "admin"
    distantPassword = "pass"
    distantServers = ()
    incomingFolder = ""
    emailMessage = """New content has been imported in your site.
The original content is available at this address: %(sourceURL)s

The new content is available here: %(newURL)s
"""

    fixedSuffix = "/content_dispatcher_tool/receive"

    overviewPath = os.path.join(os.path.dirname(__file__), 'zmi', 'overview')

    manage_options = (

        ({'label' : 'Overview',
          'action' : 'overview'
          },)
        + ActionProviderBase.manage_options
        + PropertyManager.manage_options
        + SimpleItem.manage_options
    )

    security = ClassSecurityInfo()
    overview = PageTemplateFile(overviewPath)
    security.declareProtected(CCP.ManagePortal, 'overview')

    _actions = (
        ActionInformation(
            id='dispatch',
            title='Dispatch',
            action=Expression(
                text='python:"%s/cd_dispatch" % (object.isDefaultPageInFolder() and object.getParentNode().absolute_url() or object_url)'
            ),
            condition=Expression(
                text="python: object." + id + ".isActionDisplayed(object)"
            ),
            permissions=(CCP.ReviewPortalContent,),
            category='object_buttons',
            visible=True,
        ),
    )


    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container) :
        SimpleItem.manage_afterAdd(self, item, container)

    security.declareProtected(CCP.ManagePortal, "getDistantServersAsText")
    def getDistantServersAsText(self):
        return "\n".join(self.distantServers)

    security.declarePrivate('buildURL')
    def buildURL(self, url):
        urlBlocks = list(urlparse(url + self.fixedSuffix))
        urlBlocks[1] = "%s:%s@%s" % (self.distantLogin, self.distantPassword, urlBlocks[1])
        urlToCall = urlunparse(tuple(urlBlocks))
        return urlToCall

    security.declareProtected(CCP.ManagePortal, "sendContent")
    def sendContent(self, url, params):
        """
            Send the content to one of the distant servers.
        """
        try:
            response = urllib.urlopen(self.buildURL(url), params)
        except WrongAuthenticationError:
            raise Exception("Access denied.")
        except IOError, e:
            raise Exception("IOError: %s" % e)

        result = response.read()
        result.strip()
        if result != "ok":
            e = Exception("Server error: %s" % response.info()["Bobo-Exception-Type"])
            e.serverError = response.info()["Bobo-Exception-Type"]
            raise e

    security.declarePublic('getMemberInfo')
    def getMemberInfo(self, memberId=None, info=None):
        """
        Extended version of Portal membership.getMemberInfo
        return more fields:
        - email, company...

        if memberId is NOT given, return info for logged user
        if info is given, return only this info.
        """
        mtool = getToolByName(self, 'portal_membership')
        if not memberId:
            member = mtool.getAuthenticatedMember()
        else:
            member = mtool.getMemberById(memberId)

        if member is None:
            return None

        if info:
            return member.getProperty(info)

        memberinfo = {
            'fullname'    : member.getProperty('fullname'),
            'description' : member.getProperty('description'),
            'location' : member.getProperty('location'),
            'language' : member.getProperty('language'),
            'home_page' : member.getProperty('home_page'),
            'email': member.getProperty('email'),
            'company': member.getProperty('company', ''),
            'area': member.getProperty('area', ''),
            'country': member.getProperty('country', ''),
            'job': member.getProperty('job', ''),
        }

        return memberinfo


    security.declareProtected(CCP.ReviewPortalContent, "dispatch")
    def dispatch(self, context):
        """
            Dispatch the context to the specified servers
        """

        portal_properties = getToolByName(self, "portal_properties")
        if context.portal_type in portal_properties.site_properties.typesUseViewActionInListings:
            objectUrl = context.absolute_url() + "/view"
        else:
            objectUrl = context.absolute_url()

        ## update security to allow user to be manager
        currentUser = getSecurityManager().getUser()

        try:
            userAsManager = UserAsManager(currentUser)
            newSecurityManager(self.REQUEST, userAsManager)

            ## download has to be true to enable this method returning
            ## something.
            ## zexpFile is a string coming from a IOString
            zexpFile = context.getParentNode().manage_exportObject(id=context.getId(), download=True)
        finally:
            newSecurityManager(self.REQUEST, currentUser)

        error = False
        dispatch = False
        already = False
        senderEmail = self.getMemberInfo(info="email")

        ## We only check for syntaxically correct args.
        line = 0
        for server in self.distantServers: ## this is a tuple
            line += 1
            try:
                (url, mailAddress) = server.split()
            except ValueError:
                raise RuntimeError("Content dispatcher address lines are not well formated at line %d." % line)

        for server in self.distantServers:
            (url, mailAddress) = server.split()
            dispatch = True
            params = urllib.urlencode(
                {
                    "file": zexpFile,
                    "sourceURL": objectUrl,
                    "objectId": context.getId(),
                    "fromAddr": senderEmail,
                    "toAddr": mailAddress,
                }
            )

            try:
                self.sendContent(url, params)
            except Exception, e:
                if hasattr(e, "serverError") and e.serverError == "BadRequest": ## id already in use
                    already = True
                    continue
                error = True
                LOG.error(str(e) + " (%s)" % url)

        if already:
            message = "Your content has already been dispatched. If not, please report this problem to your administrator."
        elif not dispatch:
            message = "No distant server is specified: nothing to do."
        elif error:
            message = "An error has occured. Please report this problem to your administrator."
        else:
            message = "Content successfully dispatched."
        newUrl = objectUrl + "?portal_status_message=%s" % urllib.quote(message)
        self.REQUEST.RESPONSE.redirect(newUrl)


    security.declareProtected(CCP.ManagePortal, "receive")
    def receive(self):
        """
            Receive a zexp from nother site.
        """
        request = self.REQUEST
        filename = "receivedZexp.zexp"

        cfg = getConfiguration()

        filepath = os.path.join(cfg.instancehome, 'import', filename)
        zfile = file(filepath, "wb")
        zfile.write(request["file"])
        zfile.close()

        folder = self.restrictedTraverse(self.incomingFolder)
        folder.manage_importObject(filename)

        os.remove(filepath)


        try:
            message = self.emailMessage % {
                "sourceURL": request["sourceURL"],
                "newURL": folder.absolute_url() + '/' +  request["objectId"],
            }
            headers = """From: %s
To: %s

            """ % (request["fromAddr"], request["toAddr"])
            mailhost = getToolByName(self, "MailHost")
            mailhost.send(
                headers + message,
                mto=request["toAddr"],
                mfrom=request["fromAddr"],
                subject="New content has arrived!",
            )
        except socket.error:
            ## XXX be more verbose here !
            infos = sys.exc_info()
            LOG.error("SMTP IO Error:\n%s" % ((str(infos[0]), str(infos[1]), str(infos[2])), ))

        ## don't change this, this is the marker of a well done transaction
        return "ok"

    security.declarePublic(CCP.ManagePortal, "isActionDisplayed")
    def isActionDisplayed(self, context):
        if len(self.distantServers):
            return True
        return False


InitializeClass(ContentDispatcherTool)
